#include "SDL.h"
#include "Map.h"
#include "weapons.h"
#include "explosions.h"
#include "player.h"
#include <list>
using namespace std;

#define MAX_CHARS 2
#define MAX_MISCGFX 4
#define NUM_PLAYERS 4

// Callbacks for miscgfx
#define MISCGFX_MUFFIN 0
	// ^- "Happy Muffin" power up
#define MISCGFX_HEALTH 1
	// ^- "Extra Heart" power up
#define MISCGFX_BOMB 2
	// ^- A laid bombmuffin
#define MISCGFX_EXPL 3

class CGame {
	public:
		SDL_Surface *screen;
		SDL_Surface *chars[MAX_CHARS][4]; // 1 for each direction
		SDL_Surface *miscgfx[MAX_MISCGFX];
		SDL_Event event;
		bool bDone;
		
		// Game functionality
		void menuLoop(); // draw a menu allowing the user to pick something
		void gameLoop();
		void drawMap(); // draw the 10x10 map to screen
		void drawSprites();
		void drawExplosions();
		void handleInput();
		void AIUpdate(); // move our enemies around
		void wipeOutTiles(Explosion* e); // wipe out all tiles affected by an explosion
		
		// Utility
		const bool spriteCollision(float ax, float ay, float aw, float ah, float bx, float by, float bw, float bh);
			// Checks to see if movement collides with another sprite
		const bool bombCollision(float x, float y, Uint32 team);
			// is a bomb there?
		void loadSurfaces();
		void handleBombTimers();
		void handleExplosionTimers(); // handles timeouts for explosions
		void layBombForPlayer(Uint32 player); // lays a bomb, or doesn't depending on the player
		Uint32 bombsActiveForPlayer(Uint32 player); // returns the number of bombs a player has out
		const bool canMoveTo(float x, float y, Uint32 p, bool bIsMovingRight, bool bIsMovingDown); // can you move to a certain loc? runs all the collision functions
		int tiledistance(int x1, int y1, int x2, int y2);
		
		CGame();		// initialize the game
		~CGame();
		
		// Various variables
		Uint32 frames; // for frame counting
		float timer_start;
		float timer_elapsed;
		float timer_delta; // for frame rate independence
		list <ActiveBomb*> bombs;
		list <Explosion*> explosions;
		
		Map theMap; // map "file" class, copied from CSRPG2, loads 10x10 maze with given tile "theme"
		Character players[NUM_PLAYERS];
};